#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "acidblood.h"
#include "acidfuncs.h"
#include "tracking.h"
#include "events.h"
#include "list.h"  //linktolist
#include "config.h"

#define TitleColour   "#ffff66"
#define BackgndColour "#000000"
#define Bodycolour    "#999999"
#define StampColour   "#3333ff"


struct chfiles {
	struct chfiles *prev ;   
	FILE           *file ;
	char           *channel ; 
   time_t         created;   
	struct chfiles *next ;
} ;

struct chfiles *channel_files = NULL;
void logmod_utterance(int event, struct socket_info *sinfo, char *nick, char *dest, char *info);
void releaseLogfiles(void) ;
FILE* GetFileHandle(char *channel) ;
int calcExtrasEvils( char * input) ;
void doHeader( FILE * fout, char * dest) ;


/*----------------------------------------| Init module |-------------------------------------*/
int module_init(void *module) {
  
   addtrigger(module, EV_MSG, logmod_utterance );
   
   return(0) ;
}

/*--------------------------------------| Cleanup module |-------------------------------------*/
int module_shutdown(void *module) {

   deltrigger(module, EV_MSG );
   
   releaseLogfiles();

	return(0) ;
}

/*-------------------------------------| process utterance |-------------------------------------*/
void logmod_utterance(int event, struct socket_info *sinfo, char *nick, char *dest, char *info) {

  time_t         currtime;
  char           timestring[50];
  FILE           * fout;
  
  int            newbuff;
  char           *temp1, *s, *d;
  char           *tempnick;
  int            nicksize;

  if (( fout =  GetFileHandle( dest )) == NULL) {
       privmsg(sinfo, dest, "Help I can't open my diary! \n");
       return;
  }

 /* convert < = &lt;  > = &gt;   & = &amp; */
  if ((newbuff = calcExtrasEvils( info )) != 0) {
    temp1 = malloc(newbuff+2);    
    for (s = info, d = temp1; *s; ++s) {
      switch (*s) {
          case '<':
             strcpy(d, "&lt;");  
             d += 4; 
          break;
          case '>':
             strcpy(d, "&gt;"); 
             d += 4; 
          break;
          case '&':
             strcpy(d, "&amp;"); 
             d += 5; 
          break;
          default:
             *d = *s;
             d++;
          break; 
      } 
   } 
   *d = '\0';
  } else {
    temp1 = strdup(info);  
  }  
  
  nicksize = strlen(nick);
  if (nicksize < 15) nicksize = 15; 
  tempnick = (char *)malloc(nicksize+2);
  memset(tempnick, '.', nicksize);
  *(tempnick+nicksize) = '\0';
  memcpy((tempnick+(nicksize-strlen(nick))), nick, strlen(nick));
  
   
  time(&currtime);
  strftime(timestring, 48, "%H:%M:%S", gmtime(&currtime));       
  fprintf(fout, "<font color=\"%s\"><small><small>%s:</small></small>&nbsp;%s:</font><font color=\"%s\">%s</font><br>\n", StampColour, timestring, tempnick, Bodycolour, temp1);
  free(temp1);
  free(tempnick);
  
  fflush(fout);
  

}

/*---------------------------------| Generate a log file header |-------------------------------------*/

void doHeader( FILE * fout, char * dest) {
    fprintf(fout, "<html>\n");
    fprintf(fout, "<body alink=\"#000099\" vlink=\"#990099\" link=\"#000099\" bgcolor=\"%s\" text=\"#000000\">\n", BackgndColour);
    fprintf(fout, "<div align=\"center\"><font color=\"%s\"><big><big><b>%s</b></big></big></font></div><tt>\n", TitleColour, dest);
}


/*--------------------------| calculate string length with html expansions |---------------------------*/

int calcExtrasEvils( char * input) {

  char * c;
  int  count;
  
  c = input;
  count = 0;
  
  while( *c ) {
  
     if (0) {
     } else if ((*c == '<')|(*c == '>')) {
       count += 4;
     } else if (*c == '&') {
       count += 5;
     } else {
       count ++;
     }
     c++;
  }
  
  return count;

}

/*---------------------------------------| Find log for channel |-------------------------------------*/

struct chfiles *find_logfile(char *channel)
{
	struct chfiles *logfiles ;

	logfiles = channel_files ;
	
	while(logfiles != NULL) {
		if( strncmp(channel, logfiles->channel, 20) == 0 ) {
			return(logfiles) ;
		} 
		logfiles = logfiles->next ;
	}
	return(NULL) ;	
}


/*----------------------------------------| Cleanup Linked List |-------------------------------------*/
void releaseLogfiles(void) {

	struct chfiles *logfile, *tempfile ;

	logfile = channel_files ;
	
	while(logfile != NULL) {
   
     tempfile = logfile;
     logfile  = logfile->next ;
     
      UnlinkListItem(logfile, channel_files) ;
      fclose(tempfile->file);
		free(tempfile->channel);
      free(tempfile);      
		
	}
}

/*-------------------------------| Get log filehandle for channel |-------------------------------------*/
 
FILE* GetFileHandle(char *channel) {

  struct chfiles  *logfile, *tempfile;
  time_t           currtime;
  struct tm        currtimes;
  struct tm        filetimes;
  char             temp[40];
  char             filename[512];

   time( & currtime );
   currtimes = *gmtime( & currtime);
   strftime(temp, 38, "-%d-%m-%Y", & currtimes); /* "-10-01-2005" */
  
  if ((logfile = find_logfile(channel)) != NULL) {
  
    filetimes = *gmtime( & logfile->created );
                
    if ( filetimes.tm_yday != currtimes.tm_yday ) {
        fclose(logfile->file);

        filename[0]='\0';                             /* ""                                                         */
        strncat(filename, LOG, 200);                  /* "/files/prog/src/acidblood/logs"                           */
        strcpy((strrchr(filename, '/')+1), "html/");  /* "/files/prog/src/acidblood/html/"                          */
        strncat(filename, channel, 20);               /* "/files/prog/src/acidblood/html/acidblood"                 */  
        strcat(filename, temp);                       /* "/files/prog/src/acidblood/html/acidblood-10-01-2005"      */
        strcat(filename, ".html");                    /* "/files/prog/src/acidblood/html/acidblood-10-01-2005.html" */

        if ((logfile->file = fopen(filename, "a")) == NULL) {
          return NULL;
        }
        logfile->created = currtime;
        
        if (ftell(logfile->file) == 0) {  // add headers to any new filea
          doHeader( logfile->file, channel) ;
        }
        
      }
    } else {
      filename[0]='\0';                            
      strncat(filename, LOG, 200);                  
      strcpy((strrchr(filename, '/')+1), "html/");  
      strncat(filename, channel, 20);               
      strcat(filename, temp);                      
      strcat(filename, ".html");                    

      if ((logfile = malloc(sizeof(struct chfiles))) == NULL) {
        return NULL;
      }

      logfile->created = currtime;
      logfile->channel = strdup(channel);
      if ((logfile->file = fopen(filename, "a")) == NULL) {
        free(logfile);
        return NULL;
      }
      LinkToList(logfile, tempfile, channel_files) ;	
      
      if (ftell(logfile->file) == 0) {       // add headers to any new files
        doHeader( logfile->file, channel) ;
      }
    }        
    
  return (logfile->file);
}



